@@ -48,10 +48,10 @@ sub setup()
 	'Directive'        => [$ANSI_colors{'magenta'},  $ANSI_colors{'none'}],
 	'Label'            => [$ANSI_colors{'magenta'},  $ANSI_colors{'none'}],
 	'Quote'            => [$ANSI_colors{'black'},    $ANSI_colors{'none'}],
-	'String'           => [$ANSI_colors{'bcyan'},    $ANSI_colors{'none'}],
+	'String'           => [$ANSI_colors{'gray'},    $ANSI_colors{'none'}],
 	'Subroutine'       => [$ANSI_colors{'bblue'},    $ANSI_colors{'none'}],
 	'Variable_Scalar'  => [$ANSI_colors{'bgreen'},   $ANSI_colors{'none'}],
-	'Variable_Array'   => [$ANSI_colors{'bgreen'},   $ANSI_colors{'none'}],
+	'Variable_Array'   => [$ANSI_colors{'magenta'},   $ANSI_colors{'none'}],
 	'Variable_Hash'    => [$ANSI_colors{'bgreen'},   $ANSI_colors{'none'}],
 	'Variable_Typeglob'=> [$ANSI_colors{'black'},    $ANSI_colors{'none'}],
 	'Whitespace'       => ['',                       ''                  ],
